/*
 * Main.java
 *
 * Created on 20 octobre 2004, 11:52
 */

package metacalendrier;

import java.io.*;
import java.util.logging.*;
import java.net.URL;
import java.sql.*;

import roadrunner.*;
/**
 * Toutes les fonctionalits pricipales du programme
 *
 * @author  Hai Nam
 */
public class Main 
{
    public static SQL sql;
    public static String homePathInfo = ".";
    static final public String helpMessage =
            "\n" +
            "Syntaxe:  java metacalendrier.Main <commande>\n" +
            "ou commande est une entre:\n" +
            "  reset     vider la BDD\n" +
            "  extract   extraire les evenements a partir du site web et les mettre dans\n" +
            "            output/events.xml\n" +
            "  parse     mettre les evenements (output/events.xml) en BDD\n" +
            "  refine    chercher et supprimer des doublons\n" +
            "  list      afficher tous evenements\n\n";
    
    /** Creates a new instance of Main */
    public Main() {
    }
    
    /**
     * Demonstration toutes les fonctionnalits
     * @param args the command line arguments
     */
    public static void main(String args[]) throws Exception {
        System.setProperty("rr.home", homePathInfo);
        sql = new SQL();
        String cmd = args.length==0 ? "null" : args[0];
        if (cmd.equals("reset")) {
            reset();
        } else if (cmd.equals("extract")) {
            extract();
        } else if (cmd.equals("parse")) {
            parsing();
        } else if (cmd.equals("refine")) {
            refine();
        } else if (cmd.equals("list")) {
            listEvents();
        } else if (cmd.equals("wrap")) {
            roadrunner.Shell.main("-Nevents -Oexamples/campus.acm.org/events.xml -Fexamples/acm.txt".split(" "));
        } else {
            System.out.println(helpMessage);
        }
    }
    
    /**
     * Vider les 2 tables des vnements, pour tester
     */
    public static void reset(){
        try{
            Statement stmt = sql.getConn().createStatement();
            stmt.execute("DELETE FROM " + sql.tblEvents);
            stmt.execute("DELETE FROM " + sql.tblRawEvents);
        } catch( Exception ex ){
            ex.printStackTrace();
        }
        System.out.println("Base des donnees a ete videe!");
    }
    
    /**
     * Extract information from a webpage 
     * and store it in output/output.xml
     */
    public static void extract() throws Exception {
        Extractor ext = new Extractor(Main.homePath()+"/examples/acm.wrapper.xml", "http://campus.acm.org/calendar/event_listing.cfm");
        try {
            ext.parse();
        }
        catch (Exception e){
            System.err.println("Impossible d'extraire donnees de "+ext.getUrl()+" - "+e);
        }
        System.out.println("Les evenements sont extraits et mis dans output/events.xml !");
    }
    
    /**
     * Parse XML data in output/output.xml
     * then store it into database
     */

    public static void parsing() {
        XMLParser parser = null;
        try {
            parser = new XMLParser(Main.homePath()+"/output/events.xml", sql);
        } catch (Throwable t) {
            t.printStackTrace();
        }
        parser.parse();
    }
    
    /**
     * Chercher les doublons.
     * <br/>Pour le problem d'efficacite, seulement les vnements proches
     * (en comparaisant les dates) seraient compars, on aurait la complexit
     * O(N) au lieu de O(N)
     */

    public static void refine() {
        try {
            Refiner ref = new Refiner(sql);
            ref.run();
        } catch (Throwable t) {
            t.printStackTrace();
        }
        System.out.println("Fin de chercher !");
    }
    
    /**
     * Afficher tous les vnements "officiels"
     */
    public static void listEvents(){
        ResultSet rs=null;
        Statement stmt = null;
        try{
            stmt = sql.getConn().createStatement();
        } catch( Exception ex ){
            ex.printStackTrace();
        }
         
        /**
         * Lire tous vnements
         */
        try{
            rs = stmt.executeQuery("SELECT * FROM " + sql.tblEvents + " ORDER BY dateBegin");
            while (rs.next()) {
                Event e = new Event(
                rs.getInt("id"), 
                "url", 
                new java.util.Date(), 
                rs.getDate("dateBegin"), 
                rs.getDate("dateEnd"), 
                rs.getString("place"), 
                rs.getString("country"), 
                rs.getString("title"), 
                rs.getString("detail"));

                System.out.println(e);
            }
        } catch( Exception ex ){
            ex.printStackTrace();
        }
    }
    
    public static String homePath(){
        return homePathInfo;
    }
}